<?xml version="1.0" encoding="utf-8" ?>

<!-- Transform TEI-P5 libretto to Akoma Ntoso (SayIt format) -->
<!-- by JMG & RL -->
<!-- 2014-04-15 -->

<xsl:stylesheet
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:tei="http://www.tei-c.org/ns/1.0"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://www.akomantoso.org/2.0 ./akomantoso20.xsd"
    xmlns:an="http://www.akomantoso.org/2.0"
    xmlns:html="http://www.w3.org/1999/xhtml"
    xmlns:date="http://exslt.org/dates-and-times"
    extension-element-prefixes="date">

<xsl:param name="START-TIME" />
<xsl:param name="PERFORMANCE" />

<xsl:variable name="start" select="date:add(date:date($START-TIME), date:duration((date:hour-in-day($START-TIME) * 3600) + (date:minute-in-hour($START-TIME) * 60) + date:second-in-minute($START-TIME)))" />

<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>

<xsl:key name="singers" match="//tei:singer" use="@who" />

<xsl:template match="/">
  <an:akomaNtoso>
    <an:debate name="{//tei:titleStmt/tei:title}">
      <xsl:apply-templates select="//tei:front"/>
      <xsl:apply-templates select="//tei:teiHeader"/>
      <xsl:apply-templates select="//tei:body"/>
    </an:debate>
  </an:akomaNtoso>
</xsl:template>

<xsl:template match="tei:castList">
  <an:meta>
    <an:references source="#">
      <xsl:apply-templates select=".//tei:castItem"/>
      <xsl:variable name="singers" select="//tei:singer" />
      <xsl:apply-templates select="$singers[generate-id() = generate-id(key('singers', ./@who)[1])]" mode="singer-group" />
    </an:references>
  </an:meta>
</xsl:template>

<xsl:template match="tei:castItem">
  <an:TLCPerson id="{@xml:id}" href="/ontology/person/wagner.{@xml:id}" showAs="{@xml:id}"/>
</xsl:template>

<xsl:template match="tei:singer" mode="singer-group">
  <xsl:variable name="who" select="@who" />
  <xsl:if test="not(//tei:castItem[@xml:id=$who])">
    <an:TLCPerson id="{translate(@who,' ','-')}" href="/ontology/person/wagner.{translate(@who,' ','-')}" showAs="{@who}" />
  </xsl:if>
</xsl:template>

<xsl:template match="tei:title">
  <an:preface>
    <an:docTitle>
      <xsl:value-of select="."/>
    </an:docTitle>
  </an:preface>
</xsl:template>

<xsl:template match="tei:body">
  <an:debateBody>
    <xsl:apply-templates/>
  </an:debateBody>
</xsl:template>

<xsl:template name="sum-preceding-performance-time">
  <xsl:param name="from-when" />
  <xsl:param name="acc" select="number(0)" />

  <xsl:choose>
    <xsl:when test="$from-when/@since">
      <xsl:variable name="since-id"><xsl:value-of select="substring-after($from-when/@since,'#')" /></xsl:variable>
      <xsl:call-template name="sum-preceding-performance-time">
        <xsl:with-param name="from-when" select="//tei:when[@xml:id=$since-id]" />
        <xsl:with-param name="acc" select="number($acc) + number($from-when/@interval)" />
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$acc" />
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="tei:sg[not(tei:stage)]">
  <xsl:choose>
    <xsl:when test="tei:anchor[@type='audio-cue' and contains(@xml:id,$PERFORMANCE)]">
      <xsl:variable
          name="audio-cue"
          select="tei:anchor[@type='audio-cue' and contains(@xml:id,$PERFORMANCE)][1]" />
      <xsl:variable name="audio-cue-id">
        <xsl:value-of select="$audio-cue/@xml:id" />
      </xsl:variable>
      <xsl:variable name="elapsed-time">
        <xsl:call-template name="sum-preceding-performance-time">
          <xsl:with-param name="from-when" select="//tei:when[@xml:id=concat('when-',substring-after($audio-cue/@xml:id,'cue-'))]" />
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="start-time">
        <xsl:value-of select="date:add($start, date:duration($elapsed-time))" />
      </xsl:variable>

      <an:speech by="#{translate(tei:singer[1]/@who,' ','-')}" startTime="{$start-time}">
        <xsl:apply-templates/>
      </an:speech>
    </xsl:when>
    <xsl:when test="position()=1">
      <an:speech by="#{translate(tei:singer[1]/@who,' ','-')}" startTime="$START-TIME">
        <xsl:apply-templates/>
      </an:speech>
    </xsl:when>
    <xsl:otherwise>
      <an:speech by="#{translate(tei:singer[1]/@who,' ','-')}">
        <xsl:apply-templates/>
      </an:speech>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:key name="stageKey" match="tei:sg/*[not(self::tei:singer|self::tei:stage)]"
         use="concat(generate-id(..), '|', generate-id(preceding-sibling::tei:stage[1]))" />

<xsl:template match="tei:sg[tei:stage]">
  <xsl:choose>
    <xsl:when test="tei:anchor[@type='audio-cue' and contains(@xml:id,$PERFORMANCE)]">
      <xsl:variable
          name="audio-cue"
          select="tei:anchor[@type='audio-cue' and contains(@xml:id,$PERFORMANCE)][1]" />
      <xsl:variable name="audio-cue-id">
        <xsl:value-of select="$audio-cue/@xml:id" />
      </xsl:variable>
      <xsl:variable name="elapsed-time">
        <xsl:call-template name="sum-preceding-performance-time">
          <xsl:with-param name="from-when" select="//tei:when[@xml:id=concat('when-',substring-after($audio-cue/@xml:id,'cue-'))]" />
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="start-time">
        <xsl:value-of select="date:add($start, date:duration($elapsed-time))" />
      </xsl:variable>

      <xsl:for-each select="*[generate-id() = generate-id(key('stageKey', concat(generate-id(..), '|', generate-id(preceding-sibling::tei:stage[1])))[1])]">
        <xsl:apply-templates select="preceding-sibling::tei:stage[1]" />
        <xsl:element namespace="http://www.akomantoso.org/2.0" name="speech">
          <xsl:attribute name="by">#<xsl:value-of select="translate(../tei:singer[1]/@who,' ','-')" /></xsl:attribute>
          <xsl:if test="position()=1"><xsl:attribute name="startTime"><xsl:value-of select="$start-time" /></xsl:attribute></xsl:if>
          <xsl:apply-templates select="../tei:singer[1]" />
          <xsl:apply-templates select="key('stageKey', concat(generate-id(..), '|', generate-id(preceding-sibling::tei:stage[1])))" />
        </xsl:element>
      </xsl:for-each>
    </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="first" select="position()=1" />
      <xsl:for-each select="*[generate-id() = generate-id(key('stageKey', concat(generate-id(..), '|', generate-id(preceding-sibling::tei:stage[1])))[1])]">
        <xsl:apply-templates select="preceding-sibling::tei:stage[1]" />
        <xsl:element namespace="http://www.akomantoso.org/2.0" name="speech">
          <xsl:if test="$first"><xsl:attribute name="startTime"><xsl:value-of select="$START-TIME" /></xsl:attribute></xsl:if>
          <xsl:attribute name="by">#<xsl:value-of select="translate(../tei:singer[1]/@who,' ','-')" /></xsl:attribute>
          <xsl:apply-templates select="../tei:singer[1]" />
          <xsl:apply-templates select="key('stageKey', concat(generate-id(..), '|', generate-id(preceding-sibling::tei:stage[1])))" />
        </xsl:element>
      </xsl:for-each>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="tei:stage">
  <an:narrative>
    <xsl:apply-templates/>
  </an:narrative>
</xsl:template>

<xsl:template match="tei:singer">
  <an:from>
    <xsl:apply-templates/>
  </an:from>
</xsl:template>

<xsl:template match="tei:lyric">
  <html:p>
    <xsl:apply-templates/>
  </html:p>
</xsl:template>

<xsl:template match="tei:div[@type='scene']/tei:head">
  <an:heading>
    <xsl:apply-templates/>
  </an:heading>
</xsl:template>

<xsl:template match="tei:div[@type='scene']">
  <an:debateSection name="scene" id="scene{@n}">
    <xsl:apply-templates/>
  </an:debateSection>
</xsl:template>

<xsl:template match="tei:musicalPrelude/tei:head">
  <an:subHeading>
    <xsl:apply-templates/>
  </an:subHeading>
</xsl:template>

<xsl:template match="tei:anchor[@type='audio-cue']">
  <xsl:variable name="id"><xsl:value-of select="concat('#',@xml:id)" /></xsl:variable>
  <xsl:variable name="link" select="//tei:link[contains(@target,$id)]" />
  <!-- FIXME Should we include only audio cues from the given $PERFORMANCE ? -->
  <!-- <xsl:if test="contains(@xml:id,$PERFORMANCE) and $link"> -->
  <xsl:if test="$link">
    <xsl:variable name="linkGrp" select="$link/ancestor::tei:linkGrp" />
    <xsl:variable name="release">http://musicbrainz.org/release/<xsl:value-of select="substring-after($linkGrp/@domains,'http://musicbrainz.org/release/')" /></xsl:variable>
    <xsl:variable name="track"><xsl:value-of select="substring-after(substring-after(substring-after($link/@target,'#'),'#'),'#')" /></xsl:variable>
    <html:a href="{$release}#{$track}"><xsl:value-of select="substring-after($id,'#cue-')" /></html:a>
  </xsl:if>
</xsl:template>

<xsl:template match="tei:role"/>
<xsl:template match="tei:voice"/>
<xsl:template match="tei:roleDesc"/>
<xsl:template match="tei:author"/>
<xsl:template match="tei:edition"/>
<xsl:template match="tei:name"/>
<xsl:template match="tei:resp"/>
<xsl:template match="tei:publicationStmt"/>
<xsl:template match="tei:sourceDesc"/>
<xsl:template match="tei:musicalPrelude"/>

</xsl:stylesheet>
